@@ -33,7 +33,7 @@ package Apache2::AMFLiteDetectionFilter;
   # 
 
   use vars qw($VERSION);
-  $VERSION= "4.12";;;
+  $VERSION= "4.21";;;
   my $CommonLib = new Apache2::AMFCommonLib ();
   my %MobileArray;#=$CommonLib->getMobileArray;
   my %MobileTabletArray;
@@ -367,7 +367,7 @@ sub isMobile {
   my $pair;
   my $length=0;
   foreach $pair (sort keys %MobileArray) {
-	if ($UserAgent =~ m/$pair/) {
+	if ($UserAgent =~ m/\Q$pair\E/) {
 		$isMobileValue='true';
 	}
   }
@@ -380,7 +380,7 @@ sub isTablet {
   my $pair;
   my $length=0;
   foreach $pair (sort keys %MobileTabletArray) {
-	if ($UserAgent =~ m/$pair/) {
+	if ($UserAgent =~ m/\Q$pair\E/) {
 		$isTabletValue='true';
 	}
   }
@@ -393,7 +393,7 @@ sub isTouch {
   my $pair;
   my $length=0;
   foreach $pair (sort keys %MobileTouchArray) {
-	if ($UserAgent =~ m/$pair/) {
+	if ($UserAgent =~ m/\Q$pair\E/) {
 		$isTouchValue='true';
 	}
   }
@@ -406,7 +406,7 @@ sub isTV {
   my $pair;
   my $length=0;
   foreach $pair (sort keys %MobileTVArray) {
-	if ($UserAgent =~ m/$pair/) {
+	if ($UserAgent =~ m/\Q$pair\E/) {
 		$isTVValue='true';
 	}
   }
@@ -458,6 +458,9 @@ sub getOperativeSystemVersion {
     } elsif ($os eq 'symbian') {
         $regex_param="symbianos/([0-9]\\.[0-9](\\.[0-9])?)";
         $matchOS=1;
+    } elsif ($os eq 'mac') {
+        $regex_param="os x ([0-9]([0-9]?)(_|.)([0-9]?)([0-9]?)(_?|.?)([0-9]?)([0-9]?))";
+        $matchOS=1;
     } 
     if ($matchOS == 1) {
         ($return_value) = ($UserAgent =~ /$regex_param/i);
@@ -465,6 +468,48 @@ sub getOperativeSystemVersion {
     
   return $return_value;
     
+}
+sub getOperativeDesktopSystem {
+  my ($UserAgent, $os) = (@_);
+  my $returnValue="nc";
+  my @osTypesArray = split(/,/, "windows,mac,linux");
+  my $osNumber=0;
+    foreach my $os (@osTypesArray){
+        if ($UserAgent =~ /$os/) {
+            if ($osNumber ==  0 ) {
+                $returnValue="windows";
+            } elsif ($osNumber ==  1 ) {
+                $returnValue="mac";
+            } elsif ($osNumber ==  2 ) {
+                $returnValue="windows";
+            } 
+            return $returnValue;        
+        }
+        $osNumber++;        
+    }
+    return $returnValue;
+}
+sub getBrowserVersion {
+    my ($UserAgent, $os) = (@_);
+    my $regex="(firefox|msie|chrome|chromium|safari|edge|seamonkey|opera)\\/(([0-9]?)([0-9]?)([0-9]?)(.?)([0-9]?)([0-9]?)([0-9]?)(.?)([0-9]?)([0-9]?)([0-9]?)(.?)([0-9]?)([0-9]?)([0-9]?)(.?)([0-9]?)([0-9]?)([0-9]?))";
+    my $regex2='(msie|(?!gecko.+)firefox|(?!applewebkit.+chrome.+)safari|(?!applewebkit.+)chrome|applewebkit(?!.+chrome|.+safari)|gecko(?!.+firefox))(?: |\/)([\d\.apre]+)';
+    my $type="nc";
+    my $version="nc";
+    my ($mtype,$mversion)=($UserAgent =~ /$regex/i);
+    if (!$mtype) {
+        my ($mtype2,$mversion2)=($UserAgent =~ /$regex2/i);
+        if ($mtype2) {
+            $type=$mtype2;
+            $version=$mversion2;        
+        }
+    } else {
+        $type=$mtype;
+        $version=$mversion;
+    }
+    print $type."\n";
+    print $version."\n";
+    return ($type,$version);  
+    
 }
 
 sub handler {
@@ -499,6 +544,8 @@ sub handler {
     my $version="";
     my $amf_device_os='nc';
     my $amf_device_os_version='nc';
+    my $amf_browser_type='nc';
+    my $amf_browser_version='nc';
     if ($user_agent eq "") {
 	$user_agent="no useragent found";
     }
@@ -544,13 +591,16 @@ sub handler {
 			$amf_device_istouch = &isTouch($user_agent);
 			$amf_device_istablet=&isTablet($user_agent);
             $amf_device_os=&getOperativeSystem($user_agent);
-            $amf_device_os_version=&getOperativeSystemVersion($user_agent,$amf_device_os);            
-		}
+ 		} else {
+            $amf_device_os=&getOperativeDesktopSystem($user_agent);            
+        }
+        $amf_device_os_version=&getOperativeSystemVersion($user_agent,$amf_device_os);            
 		$amf_device_istv = &isTV($user_agent);
 		if ($cookiecachesystem eq "true") {
 			$f->err_headers_out->set('Set-Cookie' => "amfID=$id; path=/;");	
 		}	
 	}
+    ($amf_browser_type,$amf_browser_version)=&getBrowserVersion($user_agent);
         if ($amfFull ne "") {
             $f->subprocess_env("AMF_FORCE_TO_DESKTOP" => 'true');
             $f->pnotes("amf_force_to_desktop" => 'true');
@@ -563,8 +613,10 @@ sub handler {
 	$f->subprocess_env("AMF_DEVICE_IS_TABLET" => $amf_device_istablet);
 	$f->subprocess_env("AMF_DEVICE_IS_TOUCH" => $amf_device_istouch);
 	$f->subprocess_env("AMF_DEVICE_IS_TV" => $amf_device_istv);
-	$f->subprocess_env("AMF_DEVICE_MOBILE_OS" => $amf_device_os);
-	$f->subprocess_env("AMF_DEVICE_MOBILE_OS_VERSION" => $amf_device_os_version);
+	$f->subprocess_env("AMF_DEVICE_OS" => $amf_device_os);
+	$f->subprocess_env("AMF_DEVICE_OS_VERSION" => $amf_device_os_version);
+	$f->subprocess_env("AMF_BROWSER_TYPE" => $amf_browser_type);
+	$f->subprocess_env("AMF_BROWSER_VERSION" => $amf_browser_version);
     
 	$f->subprocess_env("AMF_VER" => $VERSION);
 	$f->headers_out->set("AMF-Ver"=> $VERSION);
